@@ -19,7 +19,7 @@
  * Description: common stuff for bi-directional protocol ajp13/ajp14.      *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Henri Gomez <hgomez@apache.org>                            *
- * Version:     $Revision: 893085 $                                          *
+ * Version:     $Revision: 1137200 $                                          *
  ***************************************************************************/
 
 #ifndef JK_AJP_COMMON_H
@@ -70,6 +70,23 @@ extern "C"
 #define SC_A_STORED_METHOD      (unsigned char)13
 #define SC_A_ARE_DONE           (unsigned char)0xFF
 
+/*
+ * AJP private request attributes
+ *
+ * The following request attribute is recognized by Tomcat
+ * to contain the forwarded remote port.
+ */
+#define SC_A_REQ_REMOTE_PORT    ("AJP_REMOTE_PORT")
+
+/*
+ * JK public request attributes
+ *
+ * Activation state of the worker in the load balancer.
+ * Can be any of the JK_LB_ACTIVATION_TEXT_* strings
+ * from jk_lb_worker.h.
+ */
+#define SC_A_JK_LB_ACTIVATION   ("JK_LB_ACTIVATION")
+
 /*
  * Request methods, coded as numbers instead of strings.
  * The list of methods was taken from Section 5.1.1 of RFC 2616,
@@ -342,7 +359,7 @@ struct ajp_worker
 
     int retry_interval;            /*  Number of milliseconds to sleep before doing a retry */
 
-    /* 
+    /*
      * HTTP status that will cause failover (0 means disabled)
      */
      unsigned int http_status_fail_num;
@@ -365,6 +382,10 @@ struct ajp_endpoint
     jk_sock_t sd;
     int reuse;
 
+    /* Used with RECOVER_ABORT_IF_CLIENTERROR to hard abort
+       write of AJP response on client write errors */
+    int hard_close;
+
     jk_endpoint_t endpoint;
 
     jk_uint64_t left_bytes_to_send;
@@ -437,6 +458,8 @@ int JK_METHOD ajp_maintain(jk_worker_t *pThis, time_t now, jk_logger_t *l);
 
 int jk_ajp_get_cping_mode(const char *m, int def);
 
+int ajp_has_endpoint(jk_worker_t *pThis, jk_logger_t *l);
+
 #ifdef __cplusplus
 }
 #endif                          /* __cplusplus */
