@@ -23,7 +23,7 @@
  * @package jk_connect
  * @author      Gal Shachor <shachor@il.ibm.com>
  * @author      Mladen Turk <mturk@apache.org>
- * @version     $Revision: 915125 $
+ * @version     $Revision: 1125651 $
  */
 
 
@@ -55,6 +55,10 @@ static apr_pool_t *jk_apr_pool = NULL;
 #define JK_GET_SOCKET_ERRNO() ((void)0)
 #endif /* WIN32 */
 
+#if defined(NETWARE) && defined(__NOVELL_LIBC__)
+#define USE_SOCK_CLOEXEC
+#endif
+
 /* our compiler cant deal with char* <-> const char* ... */
 #if defined(NETWARE) && !defined(__NOVELL_LIBC__)
 typedef char* SET_TYPE;
@@ -71,7 +75,13 @@ typedef const char* SET_TYPE;
 static int soblock(jk_sock_t sd)
 {
 /* BeOS uses setsockopt at present for non blocking... */
-#ifndef WIN32
+#if defined (WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
+    u_long on = 0;
+    if (JK_IS_SOCKET_ERROR(ioctlsocket(sd, FIONBIO, &on))) {
+        JK_GET_SOCKET_ERRNO();
+        return errno;
+    }
+#else
     int fd_flags;
 
     fd_flags = fcntl(sd, F_GETFL, 0);
@@ -87,13 +97,7 @@ static int soblock(jk_sock_t sd)
     if (fcntl(sd, F_SETFL, fd_flags) == -1) {
         return errno;
     }
-#else
-    u_long on = 0;
-    if (JK_IS_SOCKET_ERROR(ioctlsocket(sd, FIONBIO, &on))) {
-        JK_GET_SOCKET_ERRNO();
-        return errno;
-    }
-#endif /* WIN32 */
+#endif /* WIN32 || (NETWARE && __NOVELL_LIBC__) */
     return 0;
 }
 
@@ -105,7 +109,13 @@ static int soblock(jk_sock_t sd)
  */
 static int sononblock(jk_sock_t sd)
 {
-#ifndef WIN32
+#if defined (WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
+    u_long on = 1;
+    if (JK_IS_SOCKET_ERROR(ioctlsocket(sd, FIONBIO, &on))) {
+        JK_GET_SOCKET_ERRNO();
+        return errno;
+    }
+#else
     int fd_flags;
 
     fd_flags = fcntl(sd, F_GETFL, 0);
@@ -121,13 +131,7 @@ static int sononblock(jk_sock_t sd)
     if (fcntl(sd, F_SETFL, fd_flags) == -1) {
         return errno;
     }
-#else
-    u_long on = 1;
-    if (JK_IS_SOCKET_ERROR(ioctlsocket(sd, FIONBIO, &on))) {
-        JK_GET_SOCKET_ERRNO();
-        return errno;
-    }
-#endif /* WIN32 */
+#endif /* WIN32 || (NETWARE && __NOVELL_LIBC__) */
     return 0;
 }
 
@@ -423,7 +427,7 @@ jk_sock_t jk_open_socket(struct sockaddr_in *addr, int keepalive,
                          int timeout, int connect_timeout,
                          int sock_buf, jk_logger_t *l)
 {
-    char buf[32];
+    char buf[64];
     jk_sock_t sd;
     int set = 1;
     int ret = 0;
@@ -496,7 +500,7 @@ jk_sock_t jk_open_socket(struct sockaddr_in *addr, int keepalive,
                    "failed setting SIO_KEEPALIVE_VALS (errno=%d)", errno);
             jk_close_socket(sd, l);
             JK_TRACE_EXIT(l);
-            return JK_INVALID_SOCKET;                                                
+            return JK_INVALID_SOCKET;
         }
         if (JK_IS_DEBUG_LEVEL(l))
             jk_log(l, JK_LOG_DEBUG,
@@ -627,8 +631,8 @@ iSeries when Unix98 is required at compil time */
     }
     else {
         if (JK_IS_DEBUG_LEVEL(l))
-            jk_log(l, JK_LOG_DEBUG, "socket %d connected to %s",
-                   sd, jk_dump_hinfo(addr, buf));
+            jk_log(l, JK_LOG_DEBUG, "socket %d [%s] connected",
+                   sd, jk_dump_sinfo(sd, buf));
     }
     JK_TRACE_EXIT(l);
     return sd;
@@ -668,9 +672,14 @@ int jk_close_socket(jk_sock_t sd, jk_logger_t *l)
 }
 
 #ifndef MAX_SECS_TO_LINGER
-#define MAX_SECS_TO_LINGER 30
+#define MAX_SECS_TO_LINGER 2
+#endif
+#define MS_TO_LINGER  500
+#define MS_TO_LINGER_LAST 2
+
+#ifndef MAX_LINGER_BYTES
+#define MAX_LINGER_BYTES 32768
 #endif
-#define SECONDS_TO_LINGER  2
 
 #ifndef SHUT_WR
 #ifdef SD_SEND
@@ -700,14 +709,13 @@ int jk_close_socket(jk_sock_t sd, jk_logger_t *l)
 int jk_shutdown_socket(jk_sock_t sd, jk_logger_t *l)
 {
     char dummy[512];
+    char buf[64];
+    char *sb = NULL;
     int rc = 0;
-    int rd = 0;
-    int rp = 0;
+    size_t rd = 0;
+    size_t rp = 0;
     int save_errno;
-    int timeout = SECONDS_TO_LINGER * 1000;
-#ifndef HAVE_POLL
-    struct timeval tv;
-#endif
+    int timeout = MS_TO_LINGER;
     time_t start = time(NULL);
 
     JK_TRACE_ENTER(l);
@@ -718,48 +726,31 @@ int jk_shutdown_socket(jk_sock_t sd, jk_logger_t *l)
     }
 
     save_errno = errno;
-    if (JK_IS_DEBUG_LEVEL(l))
-        jk_log(l, JK_LOG_DEBUG, "About to shutdown socket %d", sd);
-
+    if (JK_IS_DEBUG_LEVEL(l)) {
+        sb = jk_dump_sinfo(sd, buf);
+        jk_log(l, JK_LOG_DEBUG, "About to shutdown socket %d [%s]",
+               sd, sb);
+    }
     /* Shut down the socket for write, which will send a FIN
      * to the peer.
      */
     if (shutdown(sd, SHUT_WR)) {
         rc = jk_close_socket(sd, l);
         if (JK_IS_DEBUG_LEVEL(l))
-            jk_log(l, JK_LOG_DEBUG, "Failed sending SHUT_WR for socket %d", sd);
+            jk_log(l, JK_LOG_DEBUG,
+                   "Failed sending SHUT_WR for socket %d [%s]",
+                   sd, sb);
         errno = save_errno;
         JK_TRACE_EXIT(l);
         return rc;
     }
 
     do {
-#ifdef HAVE_POLL
-        struct pollfd fds;
-
-        fds.fd = sd;
-        fds.events = POLLIN;
-#else
-        fd_set rs;
-
-        FD_ZERO(&rs);
-        /* Read all data from the peer until we reach "end-of-file"
-         * (FIN from peer) or we've exceeded our overall timeout. If the
-         * backend does not send us bytes within 2 seconds
-         * (a value pulled from Apache 1.3 which seems to work well),
-         * close the connection.
-         */
-        FD_SET(sd, &rs);
-        tv.tv_sec  = timeout / 1000;
-        tv.tv_usec = (timeout % 1000) * 1000;
-#endif
         rp = 0;
-#ifdef HAVE_POLL
-        if (poll(&fds, 1, timeout) > 0)
-#else
-        if (select((int)sd + 1, &rs, NULL, NULL, &tv) > 0)
-#endif
-        {
+        if (jk_is_input_event(sd, timeout, l)) {
+            /* Do a restartable read on the socket
+             * draining out all the data currently in the socket buffer.
+             */
             do {
 #if defined(WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
                 rc = recv(sd, &dummy[0], sizeof(dummy), 0);
@@ -772,18 +763,25 @@ int jk_shutdown_socket(jk_sock_t sd, jk_logger_t *l)
                     rp += rc;
             } while (JK_IS_SOCKET_ERROR(rc) && (errno == EINTR || errno == EAGAIN));
 
-            if (rc < 0)
+            if (rc < 0) {
+                /* Read failed.
+                 * Bail out from the loop.
+                 */
                 break;
+            }
         }
-        else
+        else {
+            /* Error or timeout (reason is logged within jk_is_input_event)
+             * Exit the drain loop
+             */
             break;
+        }
         rd += rp;
         if (rp < sizeof(dummy)) {
-            if (timeout > SECONDS_TO_LINGER) {
-                /* Try once again with 1000 times smaller timeout
-                 * In our case 2 msec.
-                 */
-                timeout = SECONDS_TO_LINGER;
+            if (timeout > MS_TO_LINGER_LAST) {
+                /* Try one last time with a short timeout 
+                */
+                timeout = MS_TO_LINGER_LAST;
                 continue;
             }
             /* We have read less then size of buffer
@@ -794,25 +792,27 @@ int jk_shutdown_socket(jk_sock_t sd, jk_logger_t *l)
                 rc = jk_close_socket(sd, l);
                 if (JK_IS_DEBUG_LEVEL(l))
                     jk_log(l, JK_LOG_DEBUG,
-                           "error setting socket %d to nonblocking", sd);
+                           "error setting socket %d [%s] to nonblocking",
+                           sd, sb);
                 errno = save_errno;
                 JK_TRACE_EXIT(l);
                 return rc;
             }
             if (JK_IS_DEBUG_LEVEL(l))
                 jk_log(l, JK_LOG_DEBUG,
-                       "shutting down the read side of socket %d", sd);
+                       "shutting down the read side of socket %d [%s]",
+                       sd, sb);
             shutdown(sd, SHUT_RD);
             break;
         }
-        timeout = SECONDS_TO_LINGER * 1000;
-    } while (difftime(time(NULL), start) < MAX_SECS_TO_LINGER);
+        timeout = MS_TO_LINGER;
+    } while ((rd < MAX_LINGER_BYTES) && (difftime(time(NULL), start) < MAX_SECS_TO_LINGER));
 
     rc = jk_close_socket(sd, l);
     if (JK_IS_DEBUG_LEVEL(l))
         jk_log(l, JK_LOG_DEBUG,
-               "Shutdown socket %d and read %d lingering bytes in %d sec.",
-               sd, rd, (int)difftime(time(NULL), start));
+               "Shutdown socket %d [%s] and read %d lingering bytes in %d sec.",
+               sd, sb, rd, (int)difftime(time(NULL), start));
     errno = save_errno;
     JK_TRACE_EXIT(l);
     return rc;
@@ -853,9 +853,11 @@ int jk_tcp_socket_sendfull(jk_sock_t sd, const unsigned char *b, int len, jk_log
         } while (JK_IS_SOCKET_ERROR(wr) && (errno == EINTR || errno == EAGAIN));
 
         if (JK_IS_SOCKET_ERROR(wr)) {
+            int err;
             jk_shutdown_socket(sd, l);
+            err = (errno > 0) ? -errno : errno;
             JK_TRACE_EXIT(l);
-            return (errno > 0) ? -errno : errno;
+            return err;
         }
         else if (wr == 0) {
             jk_shutdown_socket(sd, l);
@@ -902,10 +904,10 @@ int jk_tcp_socket_recvfull(jk_sock_t sd, unsigned char *b, int len, jk_logger_t
         } while (JK_IS_SOCKET_ERROR(rd) && errno == EINTR);
 
         if (JK_IS_SOCKET_ERROR(rd)) {
-            rd = (errno > 0) ? -errno : errno;
+            int err = (errno > 0) ? -errno : errno;
             jk_shutdown_socket(sd, l);
             JK_TRACE_EXIT(l);
-            return (rd == 0) ? JK_SOCKET_EOF : rd;
+            return (err == 0) ? JK_SOCKET_EOF : err;
         }
         else if (rd == 0) {
             jk_shutdown_socket(sd, l);
@@ -935,6 +937,32 @@ char *jk_dump_hinfo(struct sockaddr_in *saddr, char *buf)
     return buf;
 }
 
+char *jk_dump_sinfo(jk_sock_t sd, char *buf)
+{
+    struct sockaddr_in rsaddr;
+    struct sockaddr_in lsaddr;
+    socklen_t          salen;
+
+    salen = sizeof(struct sockaddr);
+    if (getsockname(sd, (struct sockaddr *)&lsaddr, &salen) == 0) {
+        salen = sizeof(struct sockaddr);
+        if (getpeername(sd, (struct sockaddr *)&rsaddr, &salen) == 0) {
+            unsigned long  laddr = (unsigned  long)htonl(lsaddr.sin_addr.s_addr);
+            unsigned short lport = (unsigned short)htons(lsaddr.sin_port);
+            unsigned long  raddr = (unsigned  long)htonl(rsaddr.sin_addr.s_addr);
+            unsigned short rport = (unsigned short)htons(rsaddr.sin_port);
+            sprintf(buf, "%d.%d.%d.%d:%d -> %d.%d.%d.%d:%d",
+                    (int)(laddr >> 24), (int)((laddr >> 16) & 0xff),
+                    (int)((laddr >> 8) & 0xff), (int)(laddr & 0xff), (int)lport,
+                    (int)(raddr >> 24), (int)((raddr >> 16) & 0xff),
+                    (int)((raddr >> 8) & 0xff), (int)(raddr & 0xff), (int)rport);
+            return buf;
+        }
+    }
+    sprintf(buf, "error=%d", errno);
+    return buf;
+}
+
 /** Wait for input event on socket until timeout
  * @param sd      socket to use
  * @param timeout wait timeout in milliseconds
@@ -952,6 +980,7 @@ int jk_is_input_event(jk_sock_t sd, int timeout, jk_logger_t *l)
     struct pollfd fds;
     int rc;
     int save_errno;
+    char buf[64];
 
     JK_TRACE_ENTER(l);
 
@@ -965,6 +994,11 @@ int jk_is_input_event(jk_sock_t sd, int timeout, jk_logger_t *l)
     } while (rc < 0 && errno == EINTR);
 
     if (rc == 0) {
+        if (JK_IS_DEBUG_LEVEL(l)) {
+            jk_log(l, JK_LOG_DEBUG,
+                   "timeout during poll on socket %d [%s] (timeout=%d)",
+                   sd, jk_dump_sinfo(sd, buf), timeout);
+        }
         /* Timeout. Set the errno to timeout */
         errno = ETIMEDOUT;
         JK_TRACE_EXIT(l);
@@ -974,7 +1008,8 @@ int jk_is_input_event(jk_sock_t sd, int timeout, jk_logger_t *l)
         save_errno = errno;
         if (JK_IS_DEBUG_LEVEL(l)) {
             jk_log(l, JK_LOG_DEBUG,
-                   "error during poll on socket sd = %d (errno=%d)", sd, errno);
+                   "error during poll on socket %d [%s] (errno=%d)",
+                   sd, jk_dump_sinfo(sd, buf), errno);
         }
         errno = save_errno;
         JK_TRACE_EXIT(l);
@@ -984,12 +1019,12 @@ int jk_is_input_event(jk_sock_t sd, int timeout, jk_logger_t *l)
         save_errno = fds.revents & (POLLERR | POLLHUP);
         if (JK_IS_DEBUG_LEVEL(l)) {
             jk_log(l, JK_LOG_DEBUG,
-                   "error event during poll on socket sd = %d (event=%d)",
-               sd, save_errno);
+                   "error event during poll on socket %d [%s] (event=%d)",
+                   sd, jk_dump_sinfo(sd, buf), save_errno);
         }
         errno = save_errno;
         JK_TRACE_EXIT(l);
-        return JK_FALSE;        
+        return JK_FALSE;
     }
     errno = 0;
     JK_TRACE_EXIT(l);
@@ -1002,6 +1037,7 @@ int jk_is_input_event(jk_sock_t sd, int timeout, jk_logger_t *l)
     struct timeval tv;
     int rc;
     int save_errno;
+    char buf[64];
 
     JK_TRACE_ENTER(l);
 
@@ -1016,6 +1052,11 @@ int jk_is_input_event(jk_sock_t sd, int timeout, jk_logger_t *l)
     } while (rc < 0 && errno == EINTR);
 
     if (rc == 0) {
+        if (JK_IS_DEBUG_LEVEL(l)) {
+            jk_log(l, JK_LOG_DEBUG,
+                   "timeout during select on socket %d [%s] (timeout=%d)",
+                   sd, jk_dump_sinfo(sd, buf), timeout);
+        }
         /* Timeout. Set the errno to timeout */
 #if defined(WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
         errno = WSAETIMEDOUT - WSABASEERR;
@@ -1029,8 +1070,8 @@ int jk_is_input_event(jk_sock_t sd, int timeout, jk_logger_t *l)
         save_errno = errno;
         if (JK_IS_DEBUG_LEVEL(l)) {
             jk_log(l, JK_LOG_DEBUG,
-                   "error during select on socket sd = %d (errno=%d)",
-                   sd, errno);
+                   "error during select on socket %d [%s] (errno=%d)",
+                   sd, jk_dump_sinfo(sd, buf), errno);
         }
         errno = save_errno;
         JK_TRACE_EXIT(l);
