@@ -19,14 +19,14 @@
  * Description: Global definitions and include files that should exist     *
  *              anywhere                                                   *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
- * Version:     $Revision: 915294 $                                               *
+ * Version:     $Revision: 1126561 $                                               *
  ***************************************************************************/
 
 #ifndef JK_GLOBAL_H
 #define JK_GLOBAL_H
 
-#if !defined(WIN32) && !defined(AS400) && !defined(NETWARE)
-#include "portable.h"
+#if defined(HAVE_CONFIG_H)
+#include "config.h"
 #endif
 
 #if defined(WIN32)
@@ -116,7 +116,6 @@ extern char *strdup(const char *str);
  */
 #include <winsock2.h>
 #include <mswsock.h>
-#include <ws2tcpip.h>
 #include <mstcpip.h>
 #endif /* _WINDOWS_ */
 #ifdef _WINDOWS_
@@ -127,10 +126,11 @@ extern char *strdup(const char *str);
 #include <sys/timeb.h>
 #include <process.h>
 #include <io.h>
+#include <ws2tcpip.h>
 #else /* WIN32 */
 #include <unistd.h>
 #if defined(NETWARE) && defined(__NOVELL_LIBC__)
-#include "novsock2.h"
+#include <novsock2.h>
 #define __sys_socket_h__
 #define __netdb_h__
 #define __netinet_in_h__
@@ -141,7 +141,12 @@ extern char *strdup(const char *str);
 #include <netinet/in.h>
 #include <sys/socket.h>
 #include <fcntl.h>
-#ifndef NETWARE
+#ifdef NETWARE
+#ifndef _SOCKLEN_T
+#define _SOCKLEN_T
+typedef int socklen_t;
+#endif
+#else
 #include <netinet/tcp.h>
 #include <arpa/inet.h>
 #include <sys/un.h>
@@ -224,6 +229,7 @@ extern "C"
 
 #define JK_HTTP_OK                200
 #define JK_HTTP_BAD_REQUEST       400
+#define JK_HTTP_UNAUTHORIZED      401
 #define JK_HTTP_REQUEST_TOO_LARGE 413
 #define JK_HTTP_SERVER_ERROR      500
 #define JK_HTTP_NOT_IMPLEMENTED   501
@@ -396,3 +402,4 @@ typedef int jk_sock_t;
 #endif                          /* __cplusplus */
 
 #endif                          /* JK_GLOBAL_H */
+
