@@ -22,7 +22,7 @@
  * Author:      Mladen Turk <mturk@apache.org>                             *
  * Author:      Rainer Jung <rjung@apache.org>                             *
  * Based on:                                                               *
- * Version:     $Revision: 915149 $                                          *
+ * Version:     $Revision: 1137200 $                                          *
  ***************************************************************************/
 
 #include "jk_pool.h"
@@ -895,11 +895,11 @@ static int find_best_worker(jk_ws_service_t *s,
     return rc;
 }
 
-static lb_sub_worker_t *get_most_suitable_worker(jk_ws_service_t *s,
-                                                 lb_worker_t *p,
-                                                 char *sessionid,
-                                                 int *states,
-                                                 jk_logger_t *l)
+static int get_most_suitable_worker(jk_ws_service_t *s,
+                                    lb_worker_t *p,
+                                    char *sessionid,
+                                    int *states,
+                                    jk_logger_t *l)
 {
     int rc = -1;
     int r;
@@ -917,12 +917,12 @@ static lb_sub_worker_t *get_most_suitable_worker(jk_ws_service_t *s,
         if (JK_WORKER_USABLE_STICKY(states[0], activation)) {
             if (activation != JK_LB_ACTIVATION_DISABLED) {
                 JK_TRACE_EXIT(l);
-                return p->lb_workers;
+                return 0;
             }
         }
         else {
             JK_TRACE_EXIT(l);
-            return NULL;
+            return -1;
         }
     }
     if (p->lblock == JK_LB_LOCK_PESSIMISTIC)
@@ -969,7 +969,7 @@ static lb_sub_worker_t *get_most_suitable_worker(jk_ws_service_t *s,
                                "found worker %s (%s) for route %s and partial sessionid %s",
                                wr->name, wr->route, session_route, sessionid);
                     JK_TRACE_EXIT(l);
-                    return wr;
+                    return rc;
                 }
             }
             /* Try next partial sessionid if present */
@@ -986,7 +986,7 @@ static lb_sub_worker_t *get_most_suitable_worker(jk_ws_service_t *s,
                    "all workers are in error state for session %s",
                    session);
             JK_TRACE_EXIT(l);
-            return NULL;
+            return -1;
         }
     }
     rc = find_best_worker(s, p, states, l);
@@ -1002,10 +1002,10 @@ static lb_sub_worker_t *get_most_suitable_worker(jk_ws_service_t *s,
                    "found best worker %s (%s) using method '%s'",
                    wr->name, wr->route, jk_lb_get_method(p, l));
         JK_TRACE_EXIT(l);
-        return wr;
+        return rc;
     }
     JK_TRACE_EXIT(l);
-    return NULL;
+    return -1;
 }
 
 static void lb_add_log_items(jk_ws_service_t *s,
@@ -1085,8 +1085,18 @@ static int JK_METHOD service(jk_endpoint_t *e,
     if (p->worker->sequence != p->worker->s->h.sequence)
         jk_lb_pull(p->worker, JK_FALSE, l);
     for (i = 0; i < num_of_workers; i++) {
+        lb_sub_worker_t *rec = &(p->worker->lb_workers[i]);
+        if (rec->s->state == JK_LB_STATE_BUSY) {
+            if (ajp_has_endpoint(rec->worker, l)) {
+                if (JK_IS_DEBUG_LEVEL(l))
+                    jk_log(l, JK_LOG_DEBUG,
+                           "worker %s busy count fixed",
+                           rec->name);
+                rec->s->state = JK_LB_STATE_OK;
+            }
+        }
         /* Copy the shared state info */
-        p->states[i] = p->worker->lb_workers[i].s->state;
+        p->states[i] = rec->s->state;
     }
 
     /* set the recovery post, for LB mode */
@@ -1120,7 +1130,6 @@ static int JK_METHOD service(jk_endpoint_t *e,
                p->worker->sticky_session, sessionid ? sessionid : "empty");
 
     while (recoverable == JK_TRUE) {
-        lb_sub_worker_t *rec;
         if (attempt >= num_of_workers) {
             retry++;
             if (retry >= p->worker->retries) {
@@ -1141,20 +1150,23 @@ static int JK_METHOD service(jk_endpoint_t *e,
             }
             attempt = 0;
         }
-        rec = get_most_suitable_worker(s, p->worker, sessionid, p->states, l);
         rc = JK_FALSE;
         *is_error = JK_HTTP_SERVER_BUSY;
-        /* Do not reuse previous worker, because
-         * that worker already failed.
-         */
-        if (rec) {
+        i = get_most_suitable_worker(s, p->worker, sessionid, p->states, l);
+        if (i >= 0) {
             int r;
             int is_service_error = JK_HTTP_OK;
+            lb_sub_worker_t *rec = &(p->worker->lb_workers[i]);
             ajp_worker_t *aw = (ajp_worker_t *)rec->worker->worker_private;
             jk_endpoint_t *end = NULL;
-
+            int activation = s->extension.activation ?
+                             s->extension.activation[i] :
+                             JK_LB_ACTIVATION_UNSET;
+            if (activation == JK_LB_ACTIVATION_UNSET)
+                activation = rec->activation;
             if (!s->route)
                 s->route = rec->route;
+            s->activation = jk_lb_get_activation_direct(activation, l);
             prec = rec;
 
             if (JK_IS_DEBUG_LEVEL(l))
@@ -1422,7 +1434,7 @@ static int JK_METHOD service(jk_endpoint_t *e,
             }
         }
         else {
-            /* NULL record, no more workers left ... */
+            /* No more workers left ... */
             if (!was_forced) {
                 int nf;
                 /* Force recovery only once.
