@@ -22,7 +22,7 @@
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Dan Milstein <danmil@shore.net>                            *
  * Author:      Henri Gomez <hgomez@apache.org>                            *
- * Version:     $Revision: 745842 $                                          *
+ * Version:     $Revision: 1078846 $                                          *
  ***************************************************************************/
 
 #ifndef JK_SERVICE_H
@@ -149,6 +149,7 @@ struct jk_ws_service
     const char *protocol;
     char *req_uri;
     const char *remote_addr;
+    const char *remote_port;
     const char *remote_host;
     const char *remote_user;
     const char *auth_type;
@@ -217,6 +218,12 @@ struct jk_ws_service
      */
     const char *route;
 
+    /*
+     * Activation state of the worker in the load balancer.
+     * Will be forwarded as a request attribute.
+     */
+    const char *activation;
+
     /* Temp solution for auth. For native1 it'll be sent on each request,
        if an option is present. For native2 it'll be sent with the first
        request. On java side, both cases will work. For tomcat3.2 or
