@@ -19,7 +19,7 @@
  * Description: Shared Memory object header file                           *
  * Author:      Mladen Turk <mturk@jboss.com>                              *
  * Author:      Rainer Jung <rjung@apache.org>                             *
- * Version:     $Revision: 751916 $                                           *
+ * Version:     $Revision: 893253 $                                           *
  ***************************************************************************/
 #ifndef _JK_SHM_H
 #define _JK_SHM_H
@@ -85,6 +85,7 @@ struct jk_shm_ajp_worker
     /* Configuration data mirrored from ajp_worker */
     int cache_timeout;
     int connect_timeout;
+    int ping_timeout;
     int reply_timeout;
     int prepost_timeout;
     unsigned int recovery_opts;
@@ -94,6 +95,8 @@ struct jk_shm_ajp_worker
     /* current error state (runtime) of the worker */
     volatile int state;
     /* Statistical data */
+    /* Number of currently connected channels */
+    volatile int connected;
     /* Number of currently busy channels */
     volatile int busy;
     /* Maximum number of busy channels */
