@@ -20,7 +20,7 @@
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Henri Gomez <hgomez@apache.org>                            *
  * Author:      Rainer Jung <rjung@apache.org>                             *
- * Version:     $Revision: 915199 $                                          *
+ * Version:     $Revision: 1078846 $                                          *
  ***************************************************************************/
 
 
@@ -2043,6 +2043,7 @@ void jk_init_ws_service(jk_ws_service_t *s)
     s->protocol = NULL;
     s->req_uri = NULL;
     s->remote_addr = NULL;
+    s->remote_port = NULL;
     s->remote_host = NULL;
     s->remote_user = NULL;
     s->auth_type = NULL;
@@ -2067,6 +2068,7 @@ void jk_init_ws_service(jk_ws_service_t *s)
     s->attributes_values = NULL;
     s->num_attributes = 0;
     s->route = NULL;
+    s->activation = JK_LB_ACTIVATION_ACTIVE;
     s->secret = NULL;
     s->reco_buf = NULL;
     s->reco_status = RECO_NONE;
