@@ -17,7 +17,7 @@
 
 /***************************************************************************
  * Description: JK version header file                                     *
- * Version:     $Revision: 916931 $                                           *
+ * Version:     $Revision: 1141394 $                                           *
  ***************************************************************************/
 
 #ifndef __JK_VERSION_H
@@ -26,8 +26,7 @@
 /************** START OF AREA TO MODIFY BEFORE RELEASING *************/
 #define JK_VERMAJOR     1
 #define JK_VERMINOR     2
-#define JK_VERFIX       30
-#define JK_VERSTRING    "1.2.30"
+#define JK_VERFIX       32
 
 /* set JK_VERISRELEASE to 1 when release (do not forget to commit!) */
 #define JK_VERISRELEASE 1
@@ -38,17 +37,20 @@
 #define JK_VERRC        0
 #define JK_RCSTRING     "0"
 /* Source Control Revision as a suffix, e.g. "-r12345" */
-#define JK_REVISION "-"
+#define JK_REVISION " ()"
 
 /************** END OF AREA TO MODIFY BEFORE RELEASING *************/
 
 #if !defined(PACKAGE)
 #if defined(JK_ISAPI)
 #define PACKAGE "isapi_redirector"
+#define JK_DLL_SUFFIX "dll"
 #elif defined(JK_NSAPI)
 #define PACKAGE "nsapi_redirector"
+#define JK_DLL_SUFFIX "dll"
 #else
 #define PACKAGE "mod_jk"
+#define JK_DLL_SUFFIX "so"
 #endif
 #endif
 
@@ -68,9 +70,27 @@
 #define JK_EXPOSED_VERSION JK_EXPOSED_VERSION_INT "-dev" JK_REVISION
 #endif
 #endif
+#define JK_FULL_EXPOSED_VERSION JK_EXPOSED_VERSION JK_REVISION
 
-#define JK_MAKEVERSION(major, minor, fix, beta) (((major) << 24) + ((minor) << 16) + ((fix) << 8) + (beta))
+#define JK_MAKEVERSION(major, minor, fix, beta) \
+            (((major) << 24) + ((minor) << 16) + ((fix) << 8) + (beta))
 
 #define JK_VERSION JK_MAKEVERSION(JK_VERMAJOR, JK_VERMINOR, JK_VERFIX, JK_VERBETA)
 
+/** Properly quote a value as a string in the C preprocessor */
+#define JK_STRINGIFY(n) JK_STRINGIFY_HELPER(n)
+/** Helper macro for JK_STRINGIFY */
+#define JK_STRINGIFY_HELPER(n) #n
+#define JK_VERSTRING \
+            JK_STRINGIFY(JK_VERMAJOR) "." \
+            JK_STRINGIFY(JK_VERMINOR) "." \
+            JK_STRINGIFY(JK_VERFIX)
+
+/* macro for Win32 .rc files using numeric csv representation */
+#define JK_VERSIONCSV JK_VERMAJOR ##, \
+                    ##JK_VERMINOR ##, \
+                    ##JK_VERFIX
+
+
 #endif /* __JK_VERSION_H */
+
