@@ -18,9 +18,15 @@
 /***************************************************************************
  * Description: NSAPI plugin for Netscape servers                          *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
- * Version:     $Revision: 756058 $                                           *
+ * Version:     $Revision: 1078805 $                                           *
  ***************************************************************************/
 
+#if defined(WIN32)
+#ifndef _WIN32_WINNT
+#define _WIN32_WINNT 0x0500
+#endif
+#include <winsock2.h>
+#endif
 
 #include "nsapi.h"
 #include "jk_global.h"
@@ -119,6 +125,28 @@ static void init_workers_on_other_threads(void *init_d)
     init_on_other_thread_is_done = JK_TRUE;
 }
 
+/*
+ * Convert string to lower case.
+ * If string is longer than the provided buffer,
+ * just return the original string.
+ */
+static const char *to_lower(const char *str, char *buf, int bufsz)
+{
+    const char *from = str;
+    char *to = buf;
+    char *end = buf + (bufsz - 1);
+    while (to != end && *from) {
+        *to = (char)tolower(*from);
+        to++;
+        from++;
+    }
+    if (to != end) {
+        *to = '\0';
+        return buf;
+    }
+    return str;
+}
+
 static int JK_METHOD start_response(jk_ws_service_t *s,
                                     int status,
                                     const char *reason,
@@ -137,8 +165,16 @@ static int JK_METHOD start_response(jk_ws_service_t *s,
             param_free(pblock_remove("content-type", p->rq->srvhdrs));
 
             if (num_of_headers) {
+                /*
+                 * NSAPI expects http header names to be lower case.
+                 * This is only relevant for the server itself or other
+                 * plugins searching for headers in the pblock.
+                 * We use a buffer of limited length, because conforming
+                 * with this rule should only matter for well-known headers.
+                 */
+                char name_buf[64];
                 for (i = 0; i < (int)num_of_headers; i++) {
-                    pblock_nvinsert(header_names[i],
+                    pblock_nvinsert(to_lower(header_names[i], name_buf, 64),
                                     header_values[i], p->rq->srvhdrs);
                 }
             }
@@ -272,7 +308,7 @@ NSAPI_PUBLIC int jk_init(pblock * pb, Session * sn, Request * rq)
                              jk_parse_log_level(log_level_str))) {
         logger = NULL;
     }
-    
+
     if (jk_map_alloc(&init_map)) {
         if (jk_map_read_properties(init_map, NULL, worker_prp_file, NULL,
                                    JK_MAP_HANDLE_DUPLICATES, logger)) {
@@ -285,7 +321,7 @@ NSAPI_PUBLIC int jk_init(pblock * pb, Session * sn, Request * rq)
             }
 
             if (reject_unsafe) {
-                jk_map_add(init_map, "worker." REJECT_UNSAFE_TAG, reject_unsafe); 
+                jk_map_add(init_map, "worker." REJECT_UNSAFE_TAG, reject_unsafe);
             }
 
             jk_shm_size = jk_shm_calculate_size(init_map, logger);
@@ -307,7 +343,7 @@ NSAPI_PUBLIC int jk_init(pblock * pb, Session * sn, Request * rq)
             if (init_on_other_thread_is_done && init_on_other_thread_is_ok) {
                 magnus_atrestart(jk_term, NULL);
                 rc = REQ_PROCEED;
-                jk_log(logger, JK_LOG_INFO, "%s initialized", JK_EXPOSED_VERSION);
+                jk_log(logger, JK_LOG_INFO, "%s initialized", JK_FULL_EXPOSED_VERSION);
             }
 
 /*            if(wc_open(init_map, NULL, logger)) {
@@ -446,6 +482,8 @@ static int init_ws_service(nsapi_private_data_t * private_data,
 
     s->remote_host = session_dns(private_data->sn);
     s->remote_addr = pblock_findval("ip", private_data->sn->client);
+    /* Remote port is not available from NSAPI. */
+    s->remote_port = "0";
 
     tmp = pblock_findval("uri", private_data->rq->reqpb);
     size = 3 * strlen(tmp) + 1;
